@@ -17,7 +17,7 @@
 ## @deftypefn {Function File} [@var{numarr}, @var{txtarr}, @var{rawarr},  @var{limits}] = odsread (@var{filename})
 ## @deftypefnx {Function File} [@var{numarr}, @var{txtarr}, @var{rawarr}, @var{limits}] = odsread (@var{filename}, @var{wsh})
 ## @deftypefnx {Function File} [@var{numarr}, @var{txtarr}, @var{rawarr}, @var{limits}] = odsread (@var{filename}, @var{wsh}, @var{range})
-## @deftypefnx {Function File} [@var{numarr}, @var{txtarr}, @var{rawarr}, @var{limits}] = odsread (@var{filename}, @var{wsh}, @var{range}, @var{reqintf})
+## @deftypefnx {Function File} [@var{numarr}, @var{txtarr}, @var{rawarr}, @var{limits}, @var{extout}] = odsread (@var{filename}, @var{wsh}, @var{range}, @var{OPTS}, @dots{})
 ##
 ## Read data contained in cell range @var{range} in worksheet @var{wsh}
 ## in OpenOffice_org Calc spreadsheet file @var{filename}.  Reading
@@ -51,17 +51,39 @@
 ## argument to be @var{wsh} and to refer to a worksheet.  In that case
 ## odsread tries to read all data contained in that worksheet.
 ##
-## The optional last argument @var{reqintf} can be used to override 
-## the automatic selection by odsread of one interface out of the
-## supported ones: Java/ODFtoolkit ('OTK'), Java/jOpenDocument 
-## ('JOD'), Java/UNO bridge ('UNO'), or native Octave (OCT). Octave
-## selects one of these, preferrably in the order above, based on
-## presence of support software and the file at hand.
+## After these input arguments a number of optional arguments can be
+## supplied in any desired order:
+##
+## @table @asis
+## @item @var{Interface}
+## @var{Interface} (a three-character text sting) can be used to
+## override the automatic interface selection by odsread out of the
+## supported ones: Java/ODFtoolkit ('OTK'), Java/jOpenDocument ('JOD'),
+## Java/UNO bridge ('UNO'), or native Octave (OCT).  Octave selects one
+## of these, preferrably in the order above, based on presence of
+## support software and the file at hand.  In addition the OCT interface
+## offers .gnumeric read support.
+## @end item
+##
+## @item Function handle
+## If a function handle is specified, the pertinent function (having at
+## most two output arrays) will be applied to the numeric output data of
+## odsread.  Any second output of the function will be in a 5th output
+## argument @var{extout} of odsread.
+## @end item
+##
+## @item Options struct
+## odsread's output can be influenced to some extent by a number of
+## options.  See OPTIONS in "help ods2oct" for an overview.
+## @end itme
+## @end table
 ##
 ## Return argument @var{numarr} contains the numeric data, optional
 ## return arguments @var{txtarr} and @var{rawarr} contain text strings
 ## and the raw spreadsheet cell data, respectively, and @var{limits} is
 ## a struct containing the data origins of the various returned arrays.
+## If a function handle was specified, fifth output array @var{extout}
+## contains optional second output of the invoked function.
 ##
 ## Erroneous data and empty cells are set to NaN in @var{numarr} and
 ## turn up empty in @var{txtarr} and @var{rawarr}.  Date/time values
@@ -94,7 +116,7 @@
 ## octave-forge Java package >= 1.2.9 is required then; newer Octave versions
 ## should have Java support built-in.
 ##
-## odsread,m is just a wrapper for a collection of scripts that find out
+## odsread.m is just a wrapper for a collection of scripts that find out
 ## the interface to be used and do the actual reading.  For each call
 ## to odsread the interface must be started and the spreadsheet file read into
 ## memory.  When reading multiple ranges (in optionally multiple worksheets)
@@ -125,7 +147,7 @@
 ## Author: Philip Nienhuis <prnienhuis at users.sf.net>
 ## Created: 2009-12-12
 
-function [ numarr, txtarr, rawarr, lim ] = odsread (filename, wsh=1, datrange=[], reqintf=[])
+function [ numarr, txtarr, rawarr, lim, extout ] = odsread (filename, wsh=1, datrange=[], varargin)
 
   if (! ischar (filename))
     error ("odsread: filename (text string) expected for argument #1, not a %s\n", class (filename));
@@ -135,18 +157,44 @@ function [ numarr, txtarr, rawarr, lim ] = odsread (filename, wsh=1, datrange=[]
                        strcmpi (".gnumeric", filename(end-8:end))))
     error ("odsread: filename (incl. suffix) of a supported file type is required\n");
   endif
+  reqintf = hndl = opts = extout = [];
+
+  if (nargin > 3)
+    for ii=1:nargin-3
+      if (ischar (varargin{ii}))
+        ## Request a certain interface
+        reqintf = varargin{ii};
+      elseif (strcmp (class (varargin{ii}), "function_handle"))
+        ## Function handle to apply to output "num"
+        hndl = varargin{ii};
+      elseif (isstruct (varargin{ii}))
+        ## Various spreadsheet output options
+        opts = varargin{ii};
+      else
+        error ("odsread: illegal input arg. #%d", ii);
+      endif
+    endfor
+  endif
 
   ods = odsopen (filename, 0, reqintf);
   
   if (~isempty (ods))
 
-    [rawarr, ods, rstatus] = ods2oct (ods, wsh, datrange);
+    [rawarr, ods, rstatus] = ods2oct (ods, wsh, datrange, opts);
 
     if (rstatus)
       [numarr, txtarr, lim] = parsecell (rawarr, ods.limits);
+      if (! isempty (hndl) && ! isempty (numarr))
+        try
+          [numarr, extout] = feval (hndl, numarr);
+        catch
+          warning ("xlsread: applying specified function handle failed with:\
+error\n'%s'\n", lasterr);
+        end_try_catch
+      endif
     else
       warning (sprintf ("No data read from %s.\n", filename));
-      numarr = [];
+      rawarr = {}; numarr = []; txtarr = {}; extout = [];
     endif
   
     ods = odsclose (ods);
