@@ -49,7 +49,6 @@
 ##
 ## Notes Re: Use in Octave.
 ## -Line 83 causes a problem. Seems to work OK if commented out.FIXED
-## -Cannot find end of namelist if marker (/) is preceded by space.
 ## -Copes with Fortran comment (!) e.g.
 ## &data1
 ##   a = 100.0  ! length metres
@@ -99,8 +98,8 @@ S = struct ();
         line = [];        ##TDuell 31/03/2013 Provisional fix L.102 PRN 1apr2013
       endif
       nmlst_bdy = "";
-      if (! isempty (line));
-		    idx = strfind (line, "/");
+      if (! isempty (line))
+        idx = find_ending_idx (line);
       else
         line = "";
       endif
@@ -109,10 +108,10 @@ S = struct ();
         nmlst_bdy = [ nmlst_bdy " " line ];
         ++ii;
         line = lines{ii};
-        idx = strfind (line, "/");
+        idx = find_ending_idx (line);
       endwhile
       if (! isempty (idx) && idx(1) > 1)
-        nmlst_bdy = [ nmlst_bdy " " line ];
+        nmlst_bdy = [ nmlst_bdy " " line(1:idx(1)-1) ];
       endif
       ## Parse current namelist (set of variables)
       S.(namelst) = parse_namelist (nmlst_bdy);
@@ -122,6 +121,55 @@ S = struct ();
 endfunction
 
 
+## Internal function to find the position of the slash terminating the namelist
+function idx = find_ending_idx (strng)
+
+  ## Find all /
+  idx_all = strfind (strng, "/");
+  idx = [];
+  ## Only keep the ones not quoted
+  for ii = 1:size (idx_all, 2)
+    if (! is_quoted (strng, idx_all(ii), '"') && ...
+        ! is_quoted (strng, idx_all(ii), ''''))
+      idx = [idx idx_all(ii)];
+    endif
+  endfor
+
+endfunction
+
+
+## Internal function to check if a character at a position idx in a string
+## 'strng' is quoted by the character quote
+function ll = is_quoted (strng, idx, quote)
+
+  ## Find location of all quotes
+  idx_quotes = strfind (strng, quote);
+  if (isempty (idx_quotes))
+    ## There are no quotes
+    ll = false;
+    return
+  endif
+  ii = 1;
+  while (ii <= size (idx_quotes, 2))
+    ## Find the first quote after the character
+    if (idx_quotes(ii) > idx)
+      ## If we counted an even number of quote, this character is quoted
+      if (mod (ii, 2) == 0)
+          ll = true;
+          return
+      else
+          ll = false;
+          return
+      endif
+    endif
+    ++ii;
+  endwhile
+  ## The character is behind all quotes
+  ll = false;
+
+endfunction
+
+
 ## Internal function to parse the body text of a namelist section.
 ## Limitations: the following patterns are prohibited inside the literal
 ## strings: ".t." ".f." ".true." ".false." "(:)"
@@ -222,6 +270,7 @@ function S = parse_namelist (strng)
   endfor
 endfunction
 
+
 ## Parse the literal declarations of strings and change to Octave syntax
 function [strng, is_lit] = parse_literal_strings (strng)
 
@@ -304,3 +353,33 @@ endfunction
 %! assert (nm.test.z, [1.1+2.2i, 1i, 1], eps);
 %! assert (nm.test.y, 9+8i, eps);
 %! assert (nm.test.a, [1 2 3], eps);
+
+## Check if namelists with a whitespace before the ending / can be read
+%!test
+%! fn = tempname ();
+%! fid = fopen (fn, "w");
+%! fprintf (fid, "&test\n");
+%! fprintf (fid, " a = 1,\n");
+%! fprintf (fid, " /\n");
+%! fclose (fid);
+%! nm = read_namelist (fn);
+%! unlink (fn);
+%! assert (nm.test.a, 1, eps);
+
+## Check if character sequences can contain the / character
+%!test
+%! fn = tempname();
+%! fid = fopen (fn, "w");
+%! fprintf (fid, "&test\n");
+%! fprintf (fid, " a = '/',\n"); % inside apostrophes
+%! fprintf (fid, " b = ""/"",\n"); % inside double quotes
+%! fprintf (fid, " c = '""/',\n"); % with double quotes inside apostrophes
+%! fprintf (fid, " d = ""'/"",\n"); % with apostrophes inside double quotes
+%! fprintf (fid, "/\n");
+%! fclose (fid);
+%! nm = read_namelist (fn);
+%! unlink (fn);
+%! assert (strcmp(nm.test.a, '/'), 1, eps);
+%! assert (strcmp(nm.test.b, '/'), 1, eps);
+%! assert (strcmp(nm.test.c, '"/'), 1, eps);
+%! assert (strcmp(nm.test.d, '''/'), 1, eps);
