@@ -1,5 +1,5 @@
 // Copyright (C) 2004 Laurent Mazet <mazet@crm.mot.com>
-// Copyright (C) 2014-2017 Philip Nienhuis <prnienhuis@users.sf.net>
+// Copyright (C) 2014-2018 Philip Nienhuis <prnienhuis@users.sf.net>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU General Public License as published by the Free Software
@@ -132,7 +132,22 @@
 %! unlink (f);
 %! assert (cell2mat(s([1; 2; 3; 5; 11; 13])), [12; 22; 32; 13; 34; 15], eps);
 %! assert (s([7; 9]), {"33"; "14"});
-%! assert (all (cellfun (@isempty, s([4, 6, 8, 10, 12, 14:16]))));
+%! assert (all (cellfun (@isempty, s([6, 8, 10, 14:15]))));
+
+%% Check files with uneven line length w/o range arg
+%!test
+%! f = tmpnam ();
+%! fid = fopen (f, 'w+');
+%! fprintf (fid, 'a,b,c,d,e,f\n11,12,13,"14",15,16\n,22\n31,32,"33",34\n41,"42",,44,45\n51,52,53');
+%! fclose (fid);
+%! s = csv2cell (f);
+%! unlink (f);
+%! assert (size (s), [6 6]);
+%! assert (cell2mat(s([2, 4:6, 8:10, 12, 14, 18, 22:23, 26, 29, 32])), ...
+%!         [11, 31, 41, 51, 12, 22, 32, 52, 13, 53, 34, 44, 15, 45, 16], eps);
+%! assert ([s(1,:), s(11), s(16), s(20)], {"a", "b", "c", "d", "e", "f", ...
+%!                                         "42", "33", "14"});
+%! assert (all (cellfun (@isempty, s([3, 15, 17, 21, 24, 27:28, 30, 33:36]))));
 
 %% Check ":" separator
 %!test
@@ -172,6 +187,8 @@
 #include <octave/Cell.h>
 #include <octave/ov.h>
 
+#include "config.h"
+
 #define MAXSTRINGLENGTH 4096
 
 DEFUN_DLD (csv2cell, args, nargout, 
@@ -204,13 +221,20 @@ empty string values.  If the specified row range exceeds the number \
 of lines in the \
 file, the range is silently truncated to match the number of lines \
 per file.\n\
-Specifying a spreadsheet style cell range can be used to read csv \
-files with heterogeneous line lengths.\n\
+\n\
+If no range argument is supplied, csv2cell assumes the first line \
+contains column headers and the number of fields in the first line is \
+supposed to be the number of columns in the output array.  A warning \
+is emitted then if any data line contains more fields than the header \
+line.\n\
 \n\
 Note: newline characters in fields (i.e., multi-line text fields) are not \
 allowed.\n\
 \n\
-The maximum line width of the csv file is 4096 characters.\n\
+The maximum line width of the csv file is set to 4096 characters.\n\
+(This is also the theoretical maximum number of columns in the data \n\
+if all fields are empty, i.e., a line containing just 4096 consecutive \n\
+separators.)\n\
 @end deftypefn") {
 
   /* Get arguments */
@@ -225,12 +249,13 @@ The maximum line width of the csv file is 4096 characters.\n\
   const std::string file = args (0).string_value ();
 
   /* Look if first arg after filename = numeric. If yes => headerlines */
-  long hlines = (nargin > 1 && args(1).is_numeric_type () ? args(1).long_value() : 0);
-  int arg_sh = (nargin > 1 && args(1).is_numeric_type () ? 1 : 0);
+  long hlines = (nargin > 1 && args(1).OV_ISNUMERIC () ? args(1).long_value() : 0);
+  int arg_sh = (nargin > 1 && args(1).OV_ISNUMERIC () ? 1 : 0);
 
-  int lr = 0;
-  int lcol = 0;
+  bool lr = false;
+  int lcol = 1;
   int rcol = 0;
+  // int ipos = 1;
   int icol = 0;
   long irow = 0;
   long brow = 0;
@@ -242,63 +267,66 @@ The maximum line width of the csv file is 4096 characters.\n\
       size_t smcln = addr.find (":");
       if (addr.size () > 1 && smcln != std::string::npos)
         {
-        // Seems to be an address
-        arg_sh = 1;
-        int addr_len = addr.size ();
-        size_t ii = 0;
-        int dc;
-        long dr;
-        while (ii < addr_len)
-          {
-            switch (addr[ii])
-              {
-                case '0': case '1': case '2': case '3': case '4':
-                case '5': case '6': case '7': case '8': case '9':
-                  {
-                    irow = irow * 10;
-                    dr = (long) (addr[ii] - 48);
-                    irow = irow + dr;
-                    break;
-                  }
-                case 'A': case 'B': case 'C': case 'D': case 'E':
-                case 'F': case 'G': case 'H': case 'I': case 'J':
-                case 'K': case 'L': case 'M': case 'N': case 'O':
-                case 'P': case 'Q': case 'R': case 'S': case 'T':
-                case 'U': case 'V': case 'W': case 'X': case 'Y':
-                case 'Z':
-                  {
-                    icol = icol * 10;
-                    dc = (int) (addr[ii] - 64);
-                    icol = icol + dc;
-                    break;
-                  }
-                case ':':
-                  {
-                    lcol = icol;
-                    icol = 0;
-                    hlines = irow - 1;
-                    irow = 0;
-                    lr = 1;
-                    break;
-                  }
-                default:
+          // Seems to be an address
+          arg_sh = 1;
+          size_t addr_len = addr.size ();
+          size_t ii = 0;
+          int dc;
+          long dr;
+          while (ii < addr_len)
+            {
+              switch (addr[ii])
                 {
-                  // Assume it's a plain letter a-z
-                  error ("csv2cell: illegal character in address argument");
-                  break;
+                  case '0': case '1': case '2': case '3': case '4':
+                  case '5': case '6': case '7': case '8': case '9':
+                    {
+                      irow = irow * 10;
+                      dr = (long) (addr[ii] - 48);
+                      irow = irow + dr;
+                      break;
+                    }
+                  case 'A': case 'B': case 'C': case 'D': case 'E':
+                  case 'F': case 'G': case 'H': case 'I': case 'J':
+                  case 'K': case 'L': case 'M': case 'N': case 'O':
+                  case 'P': case 'Q': case 'R': case 'S': case 'T':
+                  case 'U': case 'V': case 'W': case 'X': case 'Y':
+                  case 'Z':
+                    {
+                      icol = icol * 26;
+                      dc = (int) (addr[ii] - 64);
+                      icol = icol + dc;
+                      break;
+                    }
+                  case ':':
+                    {
+                      lcol = icol;
+                      icol = 0;
+                      // ipos = 1;
+                      hlines = irow - 1;
+                      irow = 0;
+                      lr = true;
+                      break;
+                    }
+                  default:
+                    {
+                      // Assume it's a plain letter a-z
+                      error ("csv2cell: illegal character in address argument");
+                      break;
+                    }
                 }
-              }
-            ++ii;
-          }
+              ++ii;
+            }
         if (lcol < 1)
           lcol = 1;
         rcol = icol;
         brow = irow;
         if (rcol < lcol || brow < hlines + 1 || lcol < 1 || hlines < 0)
           error ("csv2cell: illegal cell range specified");
-        }
+        if (rcol > MAXSTRINGLENGTH + 1)
+          error ("csv2cell: nr. of columns too large for buffer size");
+      }
     }
-  
+
   const std::string _sep = (nargin > arg_sh + 1) ? args(arg_sh + 1).string_value () : ",";
   if (_sep.length() != 1)
     {
@@ -336,11 +364,11 @@ The maximum line width of the csv file is 4096 characters.\n\
 
   /* Read headerlines */
   for (long ii=0; ii < hlines; ii++)
-  {
-    fd.getline (line, MAXSTRINGLENGTH);
-    if (fd.tellg () >= fdend || fd.fail ())
-      return octave_value (Cell (0, 0));
-  }
+    {
+      fd.getline (line, MAXSTRINGLENGTH);
+      if (fd.tellg () >= fdend || fd.fail ())
+        return octave_value (Cell (0, 0));
+    }
 
   /* Read a first data line */
   str = "";
@@ -353,11 +381,10 @@ The maximum line width of the csv file is 4096 characters.\n\
     }
   str += line;
 
-  /* Parse first to get number of columns */
+  /* Parse first line to get number of columns */
   int nbcolumns = 0;
-
   for (int i = 0, len = str.length (); i <= len; i++)
-    if ((i==len) || ((str[i] == sep) || (str[i] == 10)) && (!inside))
+    if (i == len || ((str[i] == sep || str[i] == 10) && ! inside))
       nbcolumns++;
     else if ((inside) && (str[i] == prot) && ((i < len) && (str[i+1] == prot)))
       ++i;
@@ -380,9 +407,7 @@ The maximum line width of the csv file is 4096 characters.\n\
 
   /* In case of address range, fix up nbrows */
   if (lr && nbrows + hlines > brow)
-    {
       nbrows = brow - hlines;
-    }
 
   /* Rewind */
   fd.seekg (0, std::ios::beg);
@@ -394,25 +419,28 @@ The maximum line width of the csv file is 4096 characters.\n\
 
   /* Again, read headerlines */
   for (long ii = 0; ii < hlines; ii++)
-  {
-    fd.getline (line, MAXSTRINGLENGTH);
-  }
+    {
+        fd.getline (line, MAXSTRINGLENGTH);
+    }
 
   /* Read all the file until the end */
   int ncols;
   if (lr)
     ncols = rcol - lcol + 1;
   else
-    ncols = nbcolumns;
+    {
+      rcol = nbcolumns;
+      ncols = nbcolumns;
+    }
 
   Cell c (nbrows, ncols);
   // Initialize c to be able to cope with heterogeneous line length csv's
-  if (lr)
-    for (long i = 0; i < ncols; i++)
-      for (long j = 0; j < nbrows; j++)
-        c(j, i) = "";
-        // c(j, i) = octave::numeric_limits<double>::NaN ();
+  for (long i = 0; i < ncols; i++)
+    for (long j = 0; j < nbrows; j++)
+      c(j, i) = "";
+      // c(j, i) = octave::numeric_limits<double>::NaN ();
 
+  bool line_too_long = false;
   for (long i = 0; i < nbrows; i++)
     {
       /* Read a line */
@@ -433,37 +461,36 @@ The maximum line width of the csv file is 4096 characters.\n\
       int j = 0;
       // Keep track of when just read, but not yet copied field, was inside (text)
       bool oinside = false;
-      for (int k = 0, len = str.length (); k <= len; k++)
+      int len = str.length ();
+      for (int k = 0; k <= len; k++)
         {
           if (((k == len) || (str[k] == sep)) && (!inside))
             {
-              /* Check number of columns */
-              if (!lr && j == nbcolumns)
-                {
-                  fd.close ();
-                  error ("csv2cell: incorrect CSV file, line %d too long\n", i+1);
-                  return retval;
-                }
 
-              /* Check for last char to be 13 (CR) and remove if found */
-              if (word[k-1] == char(13))
+            /* Check number of columns */
+            if (!lr && !line_too_long && j == nbcolumns)
               {
-                word.resize (word.size () - 1);
+                line_too_long = true;
+                warning ("csv2cell: line(s) found with more fields than in headerline");
+                break;
               }
-              /* Check if scalar */
-              const char *word_str = word.c_str ();
-              char *err;
-              double val = strtod (word_str, &err);
-              /* Store into the cell; check if it is in address argument range*/
-              if (!lr || (lr && j+1 >= lcol && j < rcol))
-                {
-                  c(i, j-lcol+lr) = ((word == "") || oinside || (err != word_str+word.length())) ?
-                           octave_value (word) : octave_value (val);
-                }
-              j++;
-              word = "";
-              oinside = false;
-            }
+            /* Check for last char to be 13 (CR) and remove if found */
+            if (word[word.length ()-1] == char(13))
+                word.resize (word.size () - 1);
+            /* Check if scalar */
+            const char *word_str = word.c_str ();
+            char *err;
+            double val = strtod (word_str, &err);
+            /* Store into the cell; check if it is in address argument range*/
+            if ((j+1 >= lcol && j < rcol))
+              {
+                c(i, j-lcol+1) = ((word == "") || oinside || (err != word_str+word.length())) ?
+                         octave_value (word) : octave_value (val);
+              }
+            j++;
+            word = "";
+            oinside = false;
+          }
           else if ((inside) && (str[k] == prot) && ((k < len) && (str[k+1] == prot)))
             {
               /* Inside a string */
@@ -479,16 +506,6 @@ The maximum line width of the csv file is 4096 characters.\n\
           else
             word += str[k];
         }
-
-      /* Check number of columns */
-      if (!lr && j != nbcolumns)
-        {
-          fd.close ();
-          error ("csv2cell: incorrect CSV file, line %d too short\n", i+1);
-          return retval;
-        }
-      if (lr && j == ncols)
-        break;
     }
 
   /* Close file */
